# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorflow/core/protobuf/data_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorflow/core/protobuf/data_service.proto',
  package='tensorflow.data',
  syntax='proto3',
  serialized_options=_b('ZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_proto'),
  serialized_pb=_b('\n+tensorflow/core/protobuf/data_service.proto\x12\x0ftensorflow.data\"\xb7\x01\n\x11ProcessingModeDef\x12J\n\x0fsharding_policy\x18\x01 \x01(\x0e\x32\x31.tensorflow.data.ProcessingModeDef.ShardingPolicy\"V\n\x0eShardingPolicy\x12\x07\n\x03OFF\x10\x00\x12\x0b\n\x07\x44YNAMIC\x10\x01\x12\x08\n\x04\x46ILE\x10\x02\x12\x08\n\x04\x44\x41TA\x10\x03\x12\x10\n\x0c\x46ILE_OR_DATA\x10\x04\x12\x08\n\x04HINT\x10\x05\"\xe3\x01\n\x13\x44\x61taServiceMetadata\x12\x16\n\x0c\x65lement_spec\x18\x01 \x01(\x0cH\x00\x12\x45\n\x0b\x63ompression\x18\x02 \x01(\x0e\x32\x30.tensorflow.data.DataServiceMetadata.Compression\x12\x13\n\x0b\x63\x61rdinality\x18\x03 \x01(\x03\"?\n\x0b\x43ompression\x12\x1b\n\x17\x43OMPRESSION_UNSPECIFIED\x10\x00\x12\x07\n\x03OFF\x10\x01\x12\n\n\x06SNAPPY\x10\x02\x42\x17\n\x15optional_element_spec*\x88\x01\n\x0e\x44\x65ploymentMode\x12\x1f\n\x1b\x44\x45PLOYMENT_MODE_UNSPECIFIED\x10\x00\x12\x1d\n\x19\x44\x45PLOYMENT_MODE_COLOCATED\x10\x01\x12\x1a\n\x16\x44\x45PLOYMENT_MODE_REMOTE\x10\x02\x12\x1a\n\x16\x44\x45PLOYMENT_MODE_HYBRID\x10\x03\x42WZUgithub.com/tensorflow/tensorflow/tensorflow/go/core/protobuf/for_core_protos_go_protob\x06proto3')
)

_DEPLOYMENTMODE = _descriptor.EnumDescriptor(
  name='DeploymentMode',
  full_name='tensorflow.data.DeploymentMode',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DEPLOYMENT_MODE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEPLOYMENT_MODE_COLOCATED', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEPLOYMENT_MODE_REMOTE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DEPLOYMENT_MODE_HYBRID', index=3, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=481,
  serialized_end=617,
)
_sym_db.RegisterEnumDescriptor(_DEPLOYMENTMODE)

DeploymentMode = enum_type_wrapper.EnumTypeWrapper(_DEPLOYMENTMODE)
DEPLOYMENT_MODE_UNSPECIFIED = 0
DEPLOYMENT_MODE_COLOCATED = 1
DEPLOYMENT_MODE_REMOTE = 2
DEPLOYMENT_MODE_HYBRID = 3


_PROCESSINGMODEDEF_SHARDINGPOLICY = _descriptor.EnumDescriptor(
  name='ShardingPolicy',
  full_name='tensorflow.data.ProcessingModeDef.ShardingPolicy',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OFF', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DYNAMIC', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FILE', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='DATA', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='FILE_OR_DATA', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='HINT', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=162,
  serialized_end=248,
)
_sym_db.RegisterEnumDescriptor(_PROCESSINGMODEDEF_SHARDINGPOLICY)

_DATASERVICEMETADATA_COMPRESSION = _descriptor.EnumDescriptor(
  name='Compression',
  full_name='tensorflow.data.DataServiceMetadata.Compression',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='COMPRESSION_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OFF', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SNAPPY', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=390,
  serialized_end=453,
)
_sym_db.RegisterEnumDescriptor(_DATASERVICEMETADATA_COMPRESSION)


_PROCESSINGMODEDEF = _descriptor.Descriptor(
  name='ProcessingModeDef',
  full_name='tensorflow.data.ProcessingModeDef',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sharding_policy', full_name='tensorflow.data.ProcessingModeDef.sharding_policy', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _PROCESSINGMODEDEF_SHARDINGPOLICY,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=65,
  serialized_end=248,
)


_DATASERVICEMETADATA = _descriptor.Descriptor(
  name='DataServiceMetadata',
  full_name='tensorflow.data.DataServiceMetadata',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='element_spec', full_name='tensorflow.data.DataServiceMetadata.element_spec', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='compression', full_name='tensorflow.data.DataServiceMetadata.compression', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='cardinality', full_name='tensorflow.data.DataServiceMetadata.cardinality', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _DATASERVICEMETADATA_COMPRESSION,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='optional_element_spec', full_name='tensorflow.data.DataServiceMetadata.optional_element_spec',
      index=0, containing_type=None, fields=[]),
  ],
  serialized_start=251,
  serialized_end=478,
)

_PROCESSINGMODEDEF.fields_by_name['sharding_policy'].enum_type = _PROCESSINGMODEDEF_SHARDINGPOLICY
_PROCESSINGMODEDEF_SHARDINGPOLICY.containing_type = _PROCESSINGMODEDEF
_DATASERVICEMETADATA.fields_by_name['compression'].enum_type = _DATASERVICEMETADATA_COMPRESSION
_DATASERVICEMETADATA_COMPRESSION.containing_type = _DATASERVICEMETADATA
_DATASERVICEMETADATA.oneofs_by_name['optional_element_spec'].fields.append(
  _DATASERVICEMETADATA.fields_by_name['element_spec'])
_DATASERVICEMETADATA.fields_by_name['element_spec'].containing_oneof = _DATASERVICEMETADATA.oneofs_by_name['optional_element_spec']
DESCRIPTOR.message_types_by_name['ProcessingModeDef'] = _PROCESSINGMODEDEF
DESCRIPTOR.message_types_by_name['DataServiceMetadata'] = _DATASERVICEMETADATA
DESCRIPTOR.enum_types_by_name['DeploymentMode'] = _DEPLOYMENTMODE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ProcessingModeDef = _reflection.GeneratedProtocolMessageType('ProcessingModeDef', (_message.Message,), {
  'DESCRIPTOR' : _PROCESSINGMODEDEF,
  '__module__' : 'tensorflow.core.protobuf.data_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.data.ProcessingModeDef)
  })
_sym_db.RegisterMessage(ProcessingModeDef)

DataServiceMetadata = _reflection.GeneratedProtocolMessageType('DataServiceMetadata', (_message.Message,), {
  'DESCRIPTOR' : _DATASERVICEMETADATA,
  '__module__' : 'tensorflow.core.protobuf.data_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorflow.data.DataServiceMetadata)
  })
_sym_db.RegisterMessage(DataServiceMetadata)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)
