# This file is MACHINE GENERATED! Do not edit.
# Generated by: tensorflow/python/tools/api/generator/create_python_api.py script.
"""Public API for tf.config.experimental namespace.
"""

import sys as _sys

from tensorflow.python.eager.context import LogicalDeviceConfiguration as VirtualDeviceConfiguration
from tensorflow.python.framework.config import disable_mlir_bridge
from tensorflow.python.framework.config import disable_mlir_graph_optimization
from tensorflow.python.framework.config import enable_mlir_bridge
from tensorflow.python.framework.config import enable_mlir_graph_optimization
from tensorflow.python.framework.config import enable_tensor_float_32_execution
from tensorflow.python.framework.config import get_device_details
from tensorflow.python.framework.config import get_device_policy
from tensorflow.python.framework.config import get_logical_device_configuration as get_virtual_device_configuration
from tensorflow.python.framework.config import get_memory_growth
from tensorflow.python.framework.config import get_memory_info
from tensorflow.python.framework.config import get_memory_usage
from tensorflow.python.framework.config import get_synchronous_execution
from tensorflow.python.framework.config import get_visible_devices
from tensorflow.python.framework.config import list_logical_devices
from tensorflow.python.framework.config import list_physical_devices
from tensorflow.python.framework.config import reset_memory_stats
from tensorflow.python.framework.config import set_device_policy
from tensorflow.python.framework.config import set_logical_device_configuration as set_virtual_device_configuration
from tensorflow.python.framework.config import set_memory_growth
from tensorflow.python.framework.config import set_synchronous_execution
from tensorflow.python.framework.config import set_visible_devices
from tensorflow.python.framework.config import tensor_float_32_execution_enabled
from tensorflow.python.training.server_lib import ClusterDeviceFilters